/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <string.h>
#include <config.h>

#include "build_window.h"
#include "misc.h"
#include "progress_window.h"
#include "gpt.h"
#include "treeview.h"
#include "configure_window.h"

#define CFLAGS_PPRO     "-mcpu=i686 -O2"
#define CFLAGS_P4       "-mcpu=pentium4 -O2"
#define CFLAGS_AMDK7    "-mcpu=athlon -O2"

extern GptApp *the_app;

static void gpt_build_window_class_init (GptBuildWindowClass *klass);
static void gpt_build_window_init (GptBuildWindow *bw);
static void gpt_build_window_set_property (GObject      *obj,
                                           guint        prop_id,
                                           const GValue *val,
                                           GParamSpec   *pspec);
static void gpt_build_window_get_property (GObject      *obj,
                                           guint        prop_id,
                                           GValue       *val,
                                           GParamSpec   *pspec);

static void on_btn_cancel_clicked_cb (GtkWidget       *widget,
                                      GptBuildWindow  *bw);
static void on_btn_tarball_browse_clicked_cb (GtkWidget       *widget,
                                              GptBuildWindow  *bw);
static void on_btn_build_clicked_cb (GtkWidget       *widget,
                                     GptBuildWindow  *bw);
static void on_btn_patch_add_clicked_cb (GtkWidget       *widget,
                                         GptBuildWindow  *bw);
static void on_btn_patch_remove_clicked_cb (GtkWidget       *widget,
                                            GptBuildWindow  *bw);
static void on_expander_notify_cb (GObject         *object,
                                   GParamSpec      *param_spec,
                                   GptBuildWindow  *bw);
static gboolean on_entry_tarball_focus_out_cb (GtkWidget       *widget,
                                               GdkEventButton  *event,
                                               GptBuildWindow  *bw);
static void on_tv_patches_selection_changed_cb (GtkTreeSelection  *selection,
                                                GptBuildWindow    *bw);
static void on_combo_optimization_changed_cb (GtkComboBox     *combo,
                                              GptBuildWindow  *bw);

static GtkWindowClass *parent_class = NULL;

GType
gpt_build_window_get_type (void)
{
  static GType type = 0;

  if (type == 0)
    {
      static const GTypeInfo info = {
          sizeof (GptBuildWindowClass),
          NULL,   /* base_init */
          NULL,   /* base_finalize */
          (GClassInitFunc) gpt_build_window_class_init,   /* class_init */
          NULL,   /* class_finalize */
          NULL,   /* class_data */
          sizeof (GptBuildWindow),
          0,      /* n_preallocs */
          (GInstanceInitFunc) gpt_build_window_init    /* instance_init */
        };
		
      type = g_type_register_static (GTK_TYPE_WINDOW, "GptBuildWindow", &info, 0);
    }
  return type;
}

static void
gpt_build_window_class_init (GptBuildWindowClass *klass)
{
  GObjectClass *obj_class;
  obj_class = G_OBJECT_CLASS (klass);
	
  parent_class = (GtkWindowClass *) g_type_class_peek_parent (klass);
  /* obj_class->finalize = gpt_build_window_finalize; */
  obj_class->set_property = gpt_build_window_set_property;
  obj_class->get_property = gpt_build_window_get_property;
}

static void
gpt_build_window_init (GptBuildWindow *bw)
{
  g_assert (GPT_IS_BUILD_WINDOW (bw));
	
  bw->priv = g_new0 (GptBuildWindowPriv, 1);
  g_assert (bw->priv != NULL);
	
  GtkWidget *table1;
  GtkWidget *hbox1;
  GtkWidget *hbox_exp1;
  GtkWidget *label5;
  GtkWidget *label7;
  GtkWidget *label1;
  GtkWidget *hbox_exp2;
  GtkWidget *label4;
  GtkWidget *label3;
  GtkWidget *label2;
  GtkWidget *alignment1;
  GtkWidget *vbox1;
  GtkWidget *scrolledwindow1;
  GtkWidget *hbox8;
  GtkWidget *label10;
  GtkWidget *hbox10;
  GtkWidget *label9;
  GtkWidget *hbuttonbox1;
  GtkWidget *alignment2;
  GtkWidget *hbox9;
  GtkWidget *image1;
  GtkWidget *label11;
	
  gtk_window_set_title (GTK_WINDOW (bw), "Build Package from Source");
  gtk_window_set_type_hint (GTK_WINDOW (bw), GDK_WINDOW_TYPE_HINT_DIALOG);
  gtk_window_set_modal (GTK_WINDOW (bw), TRUE);
  gtk_window_set_resizable (GTK_WINDOW (bw), FALSE);
	
  table1 = gtk_table_new (9, 4, FALSE);
  gtk_widget_show (table1);
  gtk_container_add (GTK_CONTAINER (bw), table1);
	
  hbox1 = gtk_hbox_new (FALSE, 6);
  gtk_widget_show (hbox1);
  gtk_table_attach (GTK_TABLE (table1), hbox1, 2, 3, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 3);
	
  /* tarball location entry */
  bw->priv->entry_tarball = gtk_entry_new ();
  gtk_widget_show (bw->priv->entry_tarball);
  gtk_box_pack_start (GTK_BOX (hbox1),
                      bw->priv->entry_tarball,
                      TRUE, TRUE, 0);
  g_signal_connect (G_OBJECT (bw->priv->entry_tarball),
                    "focus-out-event",
                    G_CALLBACK (on_entry_tarball_focus_out_cb),
                    bw);
	
  /* tarbal browse button */
  bw->priv->btn_tarball_browse = gtk_button_new_with_mnemonic ("Browse...");
  gtk_widget_show (bw->priv->btn_tarball_browse);
  gtk_box_pack_start (GTK_BOX (hbox1), bw->priv->btn_tarball_browse, FALSE, FALSE, 0);
  g_signal_connect (G_OBJECT (bw->priv->btn_tarball_browse),
                    "clicked",
                    G_CALLBACK (on_btn_tarball_browse_clicked_cb),
                    bw);
	
  /* package name entry */
  bw->priv->entry_pkg_name = gtk_entry_new ();
  gtk_widget_show (bw->priv->entry_pkg_name);
  gtk_table_attach (GTK_TABLE (table1), bw->priv->entry_pkg_name, 2, 3, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 3);
	
  /* package version entry */
  bw->priv->entry_pkg_version = gtk_entry_new ();
  gtk_widget_show (bw->priv->entry_pkg_version);
  gtk_table_attach (GTK_TABLE (table1), bw->priv->entry_pkg_version, 2, 3, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 3);
	
  hbox_exp1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox_exp1);
  gtk_table_attach (GTK_TABLE (table1), hbox_exp1, 0, 1, 1, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox_exp1), 12);
	
  /* advanced options expander/arrow */
  bw->priv->exp_advanced = gtk_expander_new (NULL);
  g_signal_connect (bw->priv->exp_advanced,
                    "notify::expanded",
                    G_CALLBACK (on_expander_notify_cb),
                    bw);
  gtk_widget_show (bw->priv->exp_advanced);
  gtk_table_attach (GTK_TABLE (table1), bw->priv->exp_advanced, 0, 4, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 12, 12);
  label5 = gtk_label_new ("<b>Advanced Options</b>");
  gtk_widget_show (label5);
  gtk_expander_set_label_widget (GTK_EXPANDER (bw->priv->exp_advanced),
                                 label5);
  gtk_label_set_use_markup (GTK_LABEL (label5), TRUE);
	
  bw->priv->hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (bw->priv->hbox3);
  gtk_table_attach (GTK_TABLE (table1), bw->priv->hbox3, 2, 3, 5, 6,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 3);
	
  label7 = gtk_label_new (" ");
  gtk_widget_show (label7);
  gtk_box_pack_start (GTK_BOX (bw->priv->hbox3), label7, TRUE, FALSE, 0);
	
  bw->priv->combo_optimization = gtk_combo_box_new_text ();
  gtk_widget_show (bw->priv->combo_optimization);
  gtk_box_pack_start (GTK_BOX (bw->priv->hbox3), bw->priv->combo_optimization, FALSE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (bw->priv->combo_optimization), 3);
  gtk_combo_box_append_text (GTK_COMBO_BOX (bw->priv->combo_optimization), "No Optimization");
  gtk_combo_box_append_text (GTK_COMBO_BOX (bw->priv->combo_optimization), "AMD Athlon");
/*  gtk_combo_box_append_text (GTK_COMBO_BOX (bw->priv->combo_optimization), "AMD Athlon FX / Opteron"); */
  gtk_combo_box_append_text (GTK_COMBO_BOX (bw->priv->combo_optimization), "Intel Pentium PRO / II / III");
  gtk_combo_box_append_text (GTK_COMBO_BOX (bw->priv->combo_optimization), "Intel Pentium 4");
	
  gchar *opt;
  g_object_get (the_app, "optimization", &opt, NULL);
	
  if (opt)
    {
      GtkListStore *opt_store;
      opt_store = GTK_LIST_STORE (gtk_combo_box_get_model (GTK_COMBO_BOX (bw->priv->combo_optimization)));
          
      if (opt_store)
        {
          GtkTreeIter iter;
          gchar *str;
                  
          if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (opt_store), &iter))
            {
              do {
                gtk_tree_model_get (GTK_TREE_MODEL (opt_store), &iter, 0, &str, -1);
                if (str)
                  {
                    if (strcmp (opt, str) == 0)
                      {
                        gtk_combo_box_set_active_iter (
                              GTK_COMBO_BOX (bw->priv->combo_optimization),
                              &iter);
                        goto BING;
                      }
                  }
              } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (opt_store),
                                                 &iter));
            }
        }
    }
	
  gtk_combo_box_set_active (GTK_COMBO_BOX (bw->priv->combo_optimization), 0);

BING:
  g_signal_connect (bw->priv->combo_optimization,
                    "changed",
                    G_CALLBACK (on_combo_optimization_changed_cb),
                    bw);
	
  label1 = gtk_label_new ("<b>Package Details</b>");
  gtk_widget_show (label1);
  gtk_table_attach (GTK_TABLE (table1), label1, 0, 4, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 12, 12);
  gtk_label_set_use_markup (GTK_LABEL (label1), TRUE);
  gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label1), 12, 0);
	
  hbox_exp2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox_exp2);
  gtk_table_attach (GTK_TABLE (table1), hbox_exp2, 3, 4, 1, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox_exp2), 6);
	
  bw->priv->label8 = gtk_label_new ("Configure Options:");
  gtk_widget_show (bw->priv->label8);
  gtk_table_attach (GTK_TABLE (table1), bw->priv->label8, 1, 2, 6, 7,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 6, 0);
  gtk_misc_set_alignment (GTK_MISC (bw->priv->label8), 0, 0.5);
	
  bw->priv->label6 = gtk_label_new ("Optimization:");
  gtk_widget_show (bw->priv->label6);
  gtk_table_attach (GTK_TABLE (table1), bw->priv->label6, 1, 2, 5, 6,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 6, 0);
  gtk_misc_set_alignment (GTK_MISC (bw->priv->label6), 0, 0.5);
	
  label4 = gtk_label_new ("Version:");
  gtk_widget_show (label4);
  gtk_table_attach (GTK_TABLE (table1), label4, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 6, 0);
  gtk_misc_set_alignment (GTK_MISC (label4), 0, 0.5);
	
  label3 = gtk_label_new ("Package Name:");
  gtk_widget_show (label3);
  gtk_table_attach (GTK_TABLE (table1), label3, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 6, 0);
  gtk_misc_set_alignment (GTK_MISC (label3), 0, 0.5);
	
  label2 = gtk_label_new ("Source Tarball:");
  gtk_widget_show (label2);
  gtk_table_attach (GTK_TABLE (table1), label2, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 6, 0);
  gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);
	
  bw->priv->hbox5 = gtk_hbox_new (FALSE, 6);
  gtk_widget_show (bw->priv->hbox5);
  gtk_table_attach (GTK_TABLE (table1), bw->priv->hbox5, 2, 3, 6, 7,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 3);
	
  bw->priv->entry_configure_options = gtk_entry_new ();
  gtk_widget_show (bw->priv->entry_configure_options);
  gtk_box_pack_start (GTK_BOX (bw->priv->hbox5),
                      bw->priv->entry_configure_options,
                      TRUE, TRUE, 0);
	
  /* more configure options button */
  /*bw->priv->btn_configure_details = gtk_button_new_with_mnemonic ("More...");
  gtk_widget_show (bw->priv->btn_configure_details);
  gtk_box_pack_start (GTK_BOX (bw->priv->hbox5),
                      bw->priv->btn_configure_details,
                      FALSE, FALSE, 0);
  g_signal_connect (G_OBJECT (bw->priv->btn_configure_details),
                    "clicked",
                    G_CALLBACK (on_btn_configure_details_clicked_cb),
                    bw); */
	
  bw->priv->frame1 = gtk_frame_new (NULL);
  gtk_widget_show (bw->priv->frame1);
  gtk_table_attach (GTK_TABLE (table1), bw->priv->frame1, 1, 3, 7, 8,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
	
  alignment1 = gtk_alignment_new (0.5, 0.5, 1, 1);
  gtk_widget_show (alignment1);
  gtk_container_add (GTK_CONTAINER (bw->priv->frame1), alignment1);
  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment1), 0, 0, 12, 0);
	
  vbox1 = gtk_vbox_new (FALSE, 6);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (alignment1), vbox1);
  gtk_container_set_border_width (GTK_CONTAINER (vbox1), 6);
	
  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow1);
  gtk_box_pack_start (GTK_BOX (vbox1),
                      scrolledwindow1,
                      TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1),
                                  GTK_POLICY_AUTOMATIC,
                                  GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow1),
                                       GTK_SHADOW_IN);
	
  /* patches treeview */
  bw->priv->tv_patches = gtk_tree_view_new ();
  gtk_widget_show (bw->priv->tv_patches);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1),
                     bw->priv->tv_patches);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (bw->priv->tv_patches),
                                     FALSE);
  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (bw->priv->tv_patches),
                                   FALSE);
	
  /* patches list store */
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *col;
  bw->priv->patches_store = gtk_list_store_new (2,
                                                G_TYPE_STRING,
                                                G_TYPE_STRING);
  renderer = gtk_cell_renderer_text_new ();
  col = gtk_tree_view_column_new_with_attributes ("Patch", renderer,
                                                  "text", 0,
                                                  NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (bw->priv->tv_patches), col);
  gtk_tree_view_set_model (GTK_TREE_VIEW (bw->priv->tv_patches),
                           GTK_TREE_MODEL (bw->priv->patches_store));
	
  /* patches tv selection */
  GtkTreeSelection *selection;
  selection = gtk_tree_view_get_selection (
                                        GTK_TREE_VIEW (bw->priv->tv_patches));
  gtk_tree_selection_set_mode (selection,
                               GTK_SELECTION_BROWSE);
  g_signal_connect (selection,
                    "changed",
                    G_CALLBACK (on_tv_patches_selection_changed_cb),
                    bw);
	
  hbox8 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox8);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox8, FALSE, TRUE, 0);
	
  label10 = gtk_label_new (" ");
  gtk_widget_show (label10);
  gtk_box_pack_start (GTK_BOX (hbox8), label10, TRUE, TRUE, 0);
	
  hbox10 = gtk_hbox_new (TRUE, 6);
  gtk_widget_show (hbox10);
  gtk_box_pack_start (GTK_BOX (hbox8), hbox10, FALSE, TRUE, 0);
	
  /* remove patch button */
  bw->priv->btn_patch_remove = gtk_button_new_from_stock ("gtk-remove");
  gtk_widget_show (bw->priv->btn_patch_remove);
  gtk_box_pack_start (GTK_BOX (hbox10),
                      bw->priv->btn_patch_remove,
                      FALSE, FALSE, 0);
  g_signal_connect (G_OBJECT (bw->priv->btn_patch_remove),
                    "clicked",
                    G_CALLBACK (on_btn_patch_remove_clicked_cb),
                    bw);
	
  /* add patch button */
  bw->priv->btn_patch_add = gtk_button_new_from_stock ("gtk-add");
  gtk_widget_show (bw->priv->btn_patch_add);
  gtk_box_pack_start (GTK_BOX (hbox10),
                      bw->priv->btn_patch_add,
                      FALSE, TRUE, 0);
  g_signal_connect (G_OBJECT (bw->priv->btn_patch_add),
                    "clicked",
                    G_CALLBACK (on_btn_patch_add_clicked_cb),
                    bw);
	
  label9 = gtk_label_new ("Patches");
  gtk_widget_show (label9);
  gtk_frame_set_label_widget (GTK_FRAME (bw->priv->frame1), label9);
  gtk_label_set_use_markup (GTK_LABEL (label9), TRUE);
  gtk_misc_set_padding (GTK_MISC (label9), 3, 0);
	
  /* bottom buttons panel */
  hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_widget_show (hbuttonbox1);
  gtk_table_attach (GTK_TABLE (table1), hbuttonbox1, 1, 3, 8, 9,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 12);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox1), GTK_BUTTONBOX_END);
	
  /* cancel button */
  bw->priv->btn_cancel = gtk_button_new_from_stock ("gtk-cancel");
  g_signal_connect (G_OBJECT (bw->priv->btn_cancel),
                    "clicked",
                    G_CALLBACK (on_btn_cancel_clicked_cb),
                    bw);
  gtk_widget_show (bw->priv->btn_cancel);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), bw->priv->btn_cancel);
  GTK_WIDGET_SET_FLAGS (bw->priv->btn_cancel, GTK_CAN_DEFAULT);
	
  /* build button */
  bw->priv->btn_build = gtk_button_new ();
  gtk_widget_show (bw->priv->btn_build);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), bw->priv->btn_build);
  GTK_WIDGET_SET_FLAGS (bw->priv->btn_build, GTK_CAN_DEFAULT);
  g_signal_connect (G_OBJECT (bw->priv->btn_build),
                    "clicked",
                    G_CALLBACK (on_btn_build_clicked_cb),
                    bw);
	
  alignment2 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment2);
  gtk_container_add (GTK_CONTAINER (bw->priv->btn_build), alignment2);
	
  hbox9 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox9);
  gtk_container_add (GTK_CONTAINER (alignment2), hbox9);
	
  image1 = gtk_image_new_from_stock ("gtk-convert", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image1);
  gtk_box_pack_start (GTK_BOX (hbox9), image1, FALSE, FALSE, 0);
	
  label11 = gtk_label_new_with_mnemonic ("Build");
  gtk_widget_show (label11);
  gtk_box_pack_start (GTK_BOX (hbox9), label11, FALSE, FALSE, 0);
	
  on_expander_notify_cb (G_OBJECT (bw->priv->exp_advanced), NULL, bw);
}

GtkWidget *
gpt_build_window_new (void)
{
  return GTK_WIDGET (g_object_new (GPT_BUILD_WINDOW_TYPE,
                                   "type", GTK_WINDOW_TOPLEVEL,
                                   NULL));
}

static void
gpt_build_window_set_property (GObject       *obj,
                               guint         prop_id,
                               const GValue  *val,
                               GParamSpec    *pspec)
{
  GptBuildWindow *bw;
  bw = GPT_BUILD_WINDOW (obj);
	
  switch (prop_id)
    {
      default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
        break;
    }
}

static void
gpt_build_window_get_property (GObject    *obj,
                               guint      prop_id,
                               GValue     *val,
                               GParamSpec *pspec)
{
  GptBuildWindow *bw;
  bw = GPT_BUILD_WINDOW (obj);
	
  switch (prop_id) {
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
      break;
  }
}

static void
on_btn_cancel_clicked_cb (GtkWidget *widget, GptBuildWindow *bw)
{
  g_return_if_fail (GPT_IS_BUILD_WINDOW (bw));
	
  gtk_widget_destroy (GTK_WIDGET (bw));
}

static void
on_btn_tarball_browse_clicked_cb (GtkWidget *widget, GptBuildWindow *bw)
{
  GtkWidget *chooser;
  gchar *path;
  chooser = gtk_file_chooser_dialog_new ("Select Source Tarball",
                                         GTK_WINDOW(bw),
                                         GTK_FILE_CHOOSER_ACTION_OPEN,
                                         GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                         NULL);
  g_object_get (the_app, "build-package-dir", &path, NULL);
  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser),
                                       path);
	
  if (gtk_dialog_run (GTK_DIALOG (chooser)) == GTK_RESPONSE_ACCEPT)
    {
      gchar *selected_filename = gtk_file_chooser_get_filename (
                                                 GTK_FILE_CHOOSER (chooser));
      /* save selected path */
      path = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (chooser));
      g_object_set (the_app, "build-package-dir", path, NULL);
      
      gtk_entry_set_text (GTK_ENTRY (bw->priv->entry_tarball),
                          selected_filename);
      on_entry_tarball_focus_out_cb (bw->priv->entry_tarball, NULL, bw);
      
      g_free (selected_filename);
  }
  
  gtk_widget_destroy (chooser);	
}

static void
on_btn_build_clicked_cb (GtkWidget *widget, GptBuildWindow *bw)
{
  g_return_if_fail (GPT_IS_BUILD_WINDOW (bw));
  
  const gchar *tarball = gtk_entry_get_text (GTK_ENTRY (bw->priv->entry_tarball));
  const gchar *name = gtk_entry_get_text (GTK_ENTRY (bw->priv->entry_pkg_name));
  const gchar *version = gtk_entry_get_text (GTK_ENTRY (bw->priv->entry_pkg_version));
	
  if (strlen (tarball) == 0)
    {
      gpt_error (GTK_WINDOW (bw),
                 "Please specify source tarball location");
      return;
    }
  else if (strlen (name) == 0)
    {
      gpt_error (GTK_WINDOW (bw),
                 "Please specify package name");
      return;
    }
  else if (strlen (version) == 0)
    {
      gpt_error (GTK_WINDOW (bw),
                 "Please specify package version");
      return;
    }

  if (gpt_is_installed (name))
    {
      GtkWidget *dlg;
      dlg = gtk_message_dialog_new (GTK_WINDOW(bw),
                                    GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                    GTK_BUTTONS_OK_CANCEL,
                                    "Installing %s-%s will upgrade existing installation(s) of %s",
                                    name, version, name);
      gint res = gtk_dialog_run (GTK_DIALOG (dlg));
      gtk_widget_destroy (dlg);
      if (res != GTK_RESPONSE_OK)       /* we're upgrading */
          return;
    }

  /* create and initialize build progress window */
  GtkWidget *pw;
  pw = gpt_progress_window_new ();
	
  /* let the fun begin */
  gchar *cmd, *p, *str;
  cmd = g_strdup_printf ("%s/gpt-build -name=\'%s\' -version=\'%s\' -tarball=\'%s\'",
                         PACKAGE_LIBEXEC_DIR, name, version, tarball);
  
  /* iterate thru patches list */
  GtkTreeIter iter;
  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (bw->priv->patches_store),
                                     &iter))
    {
      do {
        gtk_tree_model_get (GTK_TREE_MODEL (bw->priv->patches_store),
                            &iter, 1, &str, -1);
        p = cmd;
        cmd = g_strdup_printf ("%s -patch=\'%s\'", p, str);
        g_free (p);
      } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (bw->priv->patches_store),
                                         &iter));
    }
  
  /* set optimization CFLAGS */
  g_object_get (the_app, "optimization", &str, NULL);
  if (*str)
    {
      p = cmd;
    
      if (strcmp (str, "AMD Athlon") == 0)
        {
          cmd = g_strdup_printf ("%s -opt=\'%s\'", p, CFLAGS_AMDK7);
          g_free (p);
        }
      
      else if (strcmp (str, "Intel Pentium PRO / II / III") == 0)
        {
          cmd = g_strdup_printf ("%s -opt=\'%s\'", p, CFLAGS_PPRO);
          g_free (p);
        }
      
      else if (strcmp (str, "Intel Pentium 4") == 0)
        {
          cmd = g_strdup_printf ("%s -opt=\'%s\'", p, CFLAGS_P4);
          g_free (p);
        }
    }
  
  str  = g_strdup_printf ("<big><b>Building %s v%s</b></big>\n\nSource: %s",
                                name, version, tarball);
  g_object_set (pw, "markup", str, NULL);
  g_free (str);
  
  str = g_strdup_printf ("Package %s-%s succesfully installed",
                         name, version);
  g_object_set (pw, "complete-message", str, NULL);
  g_free (str);
  
  str = g_strdup_printf ("Could not build %s", tarball);
  g_object_set (pw, "failed-message", str, NULL);
  g_free (str);
  
  str = g_strdup ("Installation aborted");
  g_object_set (pw, "aborted-message", str, NULL);
  g_free (str);

  gtk_widget_destroy (GTK_WIDGET (bw));
	
  gtk_window_set_transient_for (GTK_WINDOW (pw),
                                GTK_WINDOW (the_app->main_window));
                                
  gpt_progress_window_run (GPT_PROGRESS_WINDOW (pw), cmd);

  g_free (cmd);
}

static void
on_btn_patch_add_clicked_cb (GtkWidget *widget, GptBuildWindow *bw)
{
  g_return_if_fail (GPT_IS_BUILD_WINDOW (bw));
  
  GtkWidget *dlg;
  dlg = gtk_file_chooser_dialog_new ("Select Patch to Add",
                                     GTK_WINDOW(bw),
                                     GTK_FILE_CHOOSER_ACTION_OPEN,
                                     GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                     NULL);
  
  gchar *path;
  
  g_object_get (the_app, "patch-dir", &path, NULL);
  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dlg),
                                       path);

  if (path != NULL)
      gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dlg), path);

  if (gtk_dialog_run (GTK_DIALOG (dlg)) == GTK_RESPONSE_ACCEPT)
    {
      char *fname, *uri;

      if ((uri = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dlg))))
        {
          /* save selected path */
          path = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (dlg));
          g_object_set (the_app, "patch-dir", path, NULL);
        
          if ((fname = rindex (uri, '/')))
              fname++;
          else
              fname = uri;
                  
          GtkTreeIter iter;

          gtk_list_store_append (bw->priv->patches_store, &iter);
          gtk_list_store_set (bw->priv->patches_store, &iter, 0, fname, 1, uri, -1);
                  
          if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (bw->priv->patches_store), &iter))
            {
              GtkTreeSelection *selection;
              selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (bw->priv->tv_patches));
              gtk_tree_selection_select_iter (selection, &iter);
            }
                  
          g_free (uri);
        }
      else
          gpt_error (GTK_WINDOW (bw), "Cannot open selected patch");
    }
  
  gtk_widget_destroy (dlg);
}

static void
on_btn_patch_remove_clicked_cb (GtkWidget *widget, GptBuildWindow *bw)
{
  GtkTreeIter iter;
  GtkTreeSelection *selection;
  GtkTreeModel *model;
	
  selection = gtk_tree_view_get_selection (
                                      GTK_TREE_VIEW (bw->priv->tv_patches));
  if (gtk_tree_selection_get_selected (selection, &model, &iter))
    {
      GtkTreeIter temp_iter;
      temp_iter = iter;		
      
      if (tree_model_iter_prev (GTK_TREE_MODEL (bw->priv->patches_store),
                                &temp_iter))
          gtk_tree_selection_select_iter(selection, &temp_iter);

      else if(gtk_tree_model_iter_next(GTK_TREE_MODEL(bw->priv->patches_store),
                                       &temp_iter))
          gtk_tree_selection_select_iter(selection, &temp_iter);

      gtk_list_store_remove(bw->priv->patches_store, &iter);
    }
}

static gboolean
on_entry_tarball_focus_out_cb (GtkWidget *widget, GdkEventButton *event, GptBuildWindow *bw)
{
  g_return_val_if_fail (GPT_IS_BUILD_WINDOW (bw), FALSE);
  
  if (! GPT_ENTRY_HAS_TEXT (GTK_ENTRY (bw->priv->entry_tarball)))
      return  FALSE;
  
  gchar *version;
  gchar *name;
  gchar *p;
  gboolean got_ver = FALSE;
  
  name = g_strdup (gtk_entry_get_text (GTK_ENTRY (bw->priv->entry_tarball)));
          
  if ((version = rindex (name, '-')) != NULL)
    {
      *version = '\0';
      version++;
      p = rindex (version, '.');
      if ((( strcmp (p, ".gz") == 0) || (strcmp (p, ".bz2") == 0)) && (p != NULL))
        {
          *p = '\0';
          p = rindex (version, '.');
          if ((strcmp (p, ".tar")==0) && (p!=NULL))
            {
              *p = '\0';
              got_ver = 1;
            }
        }
    }
	
  if (rindex (name, '/') != NULL)
    {
      name = rindex (name, '/');
      name++;
    }
	
  if (got_ver && *name)
    {
      if (! GPT_ENTRY_HAS_TEXT (GTK_ENTRY(bw->priv->entry_pkg_name))) 
          gtk_entry_set_text(GTK_ENTRY(bw->priv->entry_pkg_name), name);
      if (! GPT_ENTRY_HAS_TEXT (GTK_ENTRY(bw->priv->entry_pkg_version))) 
          gtk_entry_set_text(GTK_ENTRY(bw->priv->entry_pkg_version), version);
    }

  return FALSE; /* this is somewhat important */
}

static void
on_expander_notify_cb (GObject *object, GParamSpec *param_spec, GptBuildWindow *bw)
{
  g_return_if_fail (GPT_IS_BUILD_WINDOW (bw));
  
  GtkExpander *expander;
  expander = GTK_EXPANDER (object);
  
  if (gtk_expander_get_expanded (expander) && GTK_WIDGET_IS_SENSITIVE (expander))
          gtk_widget_show_all (GTK_WIDGET (bw));
  else
    {
      gtk_widget_hide_all (bw->priv->frame1);
      gtk_widget_hide_all (bw->priv->hbox3);
      gtk_widget_hide_all (bw->priv->hbox5);
      gtk_widget_hide_all (bw->priv->label6);
      gtk_widget_hide_all (bw->priv->label8);
      
      gtk_window_resize (GTK_WINDOW (bw), 1, 1);
      gtk_window_set_resizable (GTK_WINDOW (bw), FALSE);
    }
}

static void
on_tv_patches_selection_changed_cb (GtkTreeSelection *selection, GptBuildWindow *bw)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
	

  /* Get the current row, if any */
  if (gtk_tree_selection_get_selected (selection, &model, &iter))
      gtk_widget_set_sensitive (bw->priv->btn_patch_remove, TRUE);
          
  else
      gtk_widget_set_sensitive (bw->priv->btn_patch_remove, FALSE);
}

static void
on_combo_optimization_changed_cb (GtkComboBox *combo, GptBuildWindow *bw)
{
  g_return_if_fail (GPT_IS_BUILD_WINDOW (bw));
  
  GtkListStore *store;
  GtkTreeIter iter;
  
  store = GTK_LIST_STORE (gtk_combo_box_get_model (combo));
  g_return_if_fail (GTK_IS_LIST_STORE (store));
  
  if (gtk_combo_box_get_active_iter (combo, &iter))
    {
      gchar *str;
      gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, 0, &str, -1);
      
      g_object_set (the_app, "optimization", str, NULL);
    }
}
