#include <gtk/gtk.h>

#include "details_window.h"
#include "packages.h"
#include "misc.h"
#include "gpt.h"

static void gpt_details_window_class_init (GptDetailsWindowClass *klass);
static void gpt_details_window_init (GptDetailsWindow *mw);
static void gpt_details_window_finalize (GObject *obj);
static void gpt_details_window_set_property (GObject *obj, guint prop_id, const GValue *val, GParamSpec *pspec);
/* static void gpt_details_window_get_property (GObject *obj, guint prop_id, GValue *val, GParamSpec *pspec); */
static void on_btn_close_clicked_cb (GtkWidget *widget, GptDetailsWindow *dw);

static gboolean on_window_configure_event_cb (GtkWidget *widget, GdkEvent *event, gpointer data);
static gboolean on_window_delete_event_cb (GtkWidget *widget, GdkEvent *event, gpointer data);

extern GptApp *the_app;

enum {
	PROP_PACKAGE = 1,
};

static GtkWindowClass *parent_class = NULL;

GType
gpt_details_window_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static const GTypeInfo info = {
			sizeof (GptDetailsWindowClass),
			NULL,   /* base_init */
			NULL,   /* base_finalize */
			(GClassInitFunc) gpt_details_window_class_init,   /* class_init */
			NULL,   /* class_finalize */
			NULL,   /* class_data */
			sizeof (GptDetailsWindow),
			0,      /* n_preallocs */
			(GInstanceInitFunc) gpt_details_window_init    /* instance_init */
		};
		
		type = g_type_register_static (GTK_TYPE_WINDOW, "GptDetailsWindow", &info, 0);
	}
	return type;
}

static void
gpt_details_window_class_init (GptDetailsWindowClass *klass)
{
	GObjectClass *obj_class;
	obj_class = G_OBJECT_CLASS (klass);
	
	parent_class = (GtkWindowClass*) g_type_class_peek_parent (klass);

	obj_class->finalize = gpt_details_window_finalize;
	obj_class->set_property = gpt_details_window_set_property;
	
	g_object_class_install_property (obj_class, PROP_PACKAGE,
										g_param_spec_pointer ("package",
										"Package",
										"A pointer to PkgEntry",
										G_PARAM_WRITABLE|G_PARAM_CONSTRUCT_ONLY));	
}

static void
gpt_details_window_init (GptDetailsWindow *dw)
{
	g_assert (dw != NULL);
	g_assert (GPT_IS_DETAILS_WINDOW (dw));
	
	dw->priv = g_new0 (GptDetailsWindowPriv, 1);
	
	gtk_window_set_type_hint (GTK_WINDOW (dw), GDK_WINDOW_TYPE_HINT_DIALOG);
	gtk_window_set_modal (GTK_WINDOW (dw), TRUE);

	GtkWidget *vbox3 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox3);
	gtk_container_add (GTK_CONTAINER (dw), vbox3);

	GtkWidget *hbox6 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox6);
	gtk_box_pack_start (GTK_BOX (vbox3), hbox6, FALSE, FALSE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (hbox6), 4);

	GtkWidget *label11 = gtk_label_new ("<b>Package Details</b>");
	gtk_widget_show (label11);
	gtk_box_pack_start (GTK_BOX (vbox3), label11, FALSE, FALSE, 0);
	gtk_label_set_use_markup (GTK_LABEL (label11), TRUE);
	gtk_misc_set_alignment (GTK_MISC (label11), 0, 1);
	gtk_misc_set_padding (GTK_MISC (label11), 12, 4);

	GtkWidget *table2 = gtk_table_new (4, 3, FALSE);
	gtk_widget_show (table2);
	gtk_box_pack_start (GTK_BOX (vbox3), table2, FALSE, TRUE, 0);

	dw->priv->label_size = gtk_label_new ("n/a (size)");
	gtk_widget_show (dw->priv->label_size);
	gtk_table_attach (GTK_TABLE (table2), dw->priv->label_size, 2, 3, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 4, 2);
	GTK_WIDGET_SET_FLAGS (dw->priv->label_size, GTK_CAN_FOCUS);
	gtk_label_set_use_markup (GTK_LABEL (dw->priv->label_size), TRUE);
	gtk_label_set_selectable (GTK_LABEL (dw->priv->label_size), TRUE);
	gtk_misc_set_alignment (GTK_MISC (dw->priv->label_size), 0, 0.5);

	dw->priv->label_version = gtk_label_new ("n/a (ver)");
	gtk_widget_show (dw->priv->label_version);
	gtk_table_attach (GTK_TABLE (table2), dw->priv->label_version, 2, 3, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 4, 2);
	GTK_WIDGET_SET_FLAGS (dw->priv->label_version, GTK_CAN_FOCUS);
	gtk_label_set_use_markup (GTK_LABEL (dw->priv->label_version), TRUE);
	gtk_label_set_selectable (GTK_LABEL (dw->priv->label_version), TRUE);
	gtk_misc_set_alignment (GTK_MISC (dw->priv->label_version), 0, 0.5);

	dw->priv->label_name = gtk_label_new ("n/a (name)");
	gtk_widget_show (dw->priv->label_name);
	gtk_table_attach (GTK_TABLE (table2), dw->priv->label_name, 2, 3, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 4, 2);
	GTK_WIDGET_SET_FLAGS (dw->priv->label_name, GTK_CAN_FOCUS);
	gtk_label_set_use_markup (GTK_LABEL (dw->priv->label_name), TRUE);
	gtk_label_set_selectable (GTK_LABEL (dw->priv->label_name), TRUE);
	gtk_misc_set_alignment (GTK_MISC (dw->priv->label_name), 0, 0.5);

	GtkWidget *label14 = gtk_label_new ("Version:");
	gtk_widget_show (label14);
	gtk_table_attach (GTK_TABLE (table2), label14, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label14), 0, 0.5);

	GtkWidget *label15 = gtk_label_new ("Size on Disc:");
	gtk_widget_show (label15);
	gtk_table_attach (GTK_TABLE (table2), label15, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label15), 0, 0.5);

	GtkWidget *hbox7 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox7);
	gtk_table_attach (GTK_TABLE (table2), hbox7, 0, 1, 0, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (hbox7), 15); 
	
	GtkWidget *label12 = gtk_label_new ("Package Name:");
	gtk_widget_show (label12);
	gtk_table_attach (GTK_TABLE (table2), label12, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label12), 0, 0.5);

	GtkWidget *hbox18 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox18);
	gtk_table_attach (GTK_TABLE (table2), hbox18, 3, 4, 0, 4,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (hbox18), 10);

	GtkWidget *hbox9 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox9);
	gtk_box_pack_start (GTK_BOX (vbox3), hbox9, FALSE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (hbox9), 6);

	GtkWidget *label20 = gtk_label_new ("<b>File List</b>");
	gtk_widget_show (label20);
	gtk_box_pack_start (GTK_BOX (vbox3), label20, FALSE, FALSE, 0);
	gtk_label_set_use_markup (GTK_LABEL (label20), TRUE);
	gtk_misc_set_alignment (GTK_MISC (label20), 0, 1);
	gtk_misc_set_padding (GTK_MISC (label20), 12, 0);

	GtkWidget *hbox10 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox10);
	gtk_box_pack_start (GTK_BOX (vbox3), hbox10, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (hbox10), 6);

	GtkWidget *hbox17 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox17);
	gtk_box_pack_start (GTK_BOX (hbox10), hbox17, FALSE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (hbox17), 12);
  
	GtkWidget *scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow2);
	gtk_box_pack_start (GTK_BOX (hbox10), scrolledwindow2, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow2), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow2), GTK_SHADOW_IN);

	dw->priv->file_list = gtk_tree_view_new ();
	gtk_widget_show (dw->priv->file_list);
	gtk_container_add (GTK_CONTAINER (scrolledwindow2), dw->priv->file_list);
	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (dw->priv->file_list), FALSE);
	
	GtkWidget *hbox12 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox12);
	gtk_box_pack_start (GTK_BOX (hbox10), hbox12, FALSE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (hbox12), 3);

	GtkWidget *hbox15 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox15);
	gtk_box_pack_start (GTK_BOX (vbox3), hbox15, FALSE, TRUE, 0);

	GtkWidget *hbox15_1 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox15_1);
	gtk_box_pack_start (GTK_BOX (hbox15), hbox15_1, FALSE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (hbox15_1), 6);

	GtkWidget *hseparator2 = gtk_hseparator_new ();
	gtk_widget_show (hseparator2);
	gtk_box_pack_start (GTK_BOX (hbox15), hseparator2, TRUE, TRUE, 0);
	
	GtkWidget *hbox15_2 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox15_2);
	gtk_box_pack_start (GTK_BOX (hbox15), hbox15_2, FALSE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (hbox15_2), 6);

	GtkWidget *hbox16 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox16);
	gtk_box_pack_start (GTK_BOX (vbox3), hbox16, FALSE, TRUE, 0);
	
	GtkWidget *vbox6 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox6);
	gtk_box_pack_start (GTK_BOX (hbox16), vbox6, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (vbox6), 6);

	dw->priv->btn_close = gtk_button_new_from_stock ("gtk-close");
	gtk_widget_show (dw->priv->btn_close);
	gtk_box_pack_start (GTK_BOX (hbox16), dw->priv->btn_close, FALSE, FALSE, 0);
	g_signal_connect(G_OBJECT(dw->priv->btn_close), "clicked", G_CALLBACK(on_btn_close_clicked_cb), dw);

	GtkWidget *vbox4 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox4);
	gtk_box_pack_start (GTK_BOX (hbox16), vbox4, FALSE, TRUE, 6);

	GtkWidget *vbox5 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox5);
	gtk_box_pack_start (GTK_BOX (vbox3), vbox5, FALSE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (vbox5), 6);
	
	/* set up treeview */
	GtkTreeViewColumn *column;
	GtkCellRenderer *renderer;
	GtkTreeSelection *selection;

	dw->priv->file_list_store = gtk_list_store_new (1, G_TYPE_STRING);
	gtk_tree_view_set_model (GTK_TREE_VIEW (dw->priv->file_list), GTK_TREE_MODEL (dw->priv->file_list_store));

	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("File", renderer, "text", 0, NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (dw->priv->file_list), column);

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dw->priv->file_list));
	gtk_tree_selection_set_mode (selection, GTK_SELECTION_NONE);

	gint w, h;
	g_object_get (the_app, "details-window-width", &w, NULL);
	g_object_get (the_app, "details-window-height", &h, NULL);
	gtk_window_resize (GTK_WINDOW (dw), w, h);
	
	g_signal_connect (G_OBJECT (dw), "configure-event", G_CALLBACK (on_window_configure_event_cb), NULL);
	g_signal_connect (G_OBJECT (dw), "delete-event", G_CALLBACK (on_window_delete_event_cb), NULL);
}

static void
gpt_details_window_finalize (GObject *obj)
{
	g_assert (GPT_IS_DETAILS_WINDOW (obj));

	gtk_list_store_clear (GPT_DETAILS_WINDOW (obj)->priv->file_list_store);
	
	(G_OBJECT_CLASS (parent_class)->finalize) (obj);
}

static void
gpt_details_window_set_property (GObject *obj, guint prop_id, const GValue *val, GParamSpec *pspec)
{
	GptDetailsWindow *dw;
	dw = GPT_DETAILS_WINDOW (obj);
	
	if (prop_id == PROP_PACKAGE) {
		PkgEntry *entry = g_value_get_pointer (val);
		g_return_if_fail (entry != NULL);
		g_return_if_fail (IS_PKG_ENTRY (entry));

		GtkTreeIter iter;
		GList *l;
		gchar *fname;
	
		gtk_list_store_clear (dw->priv->file_list_store);
 
		for (l = entry->files; l; l = l->next) {
			gtk_list_store_append (dw->priv->file_list_store, &iter);
			fname = g_strdup_printf ("/%s", (gchar *) l->data);
			gtk_list_store_set (dw->priv->file_list_store, &iter, 0, fname, -1);
			g_free (fname);
		}
	
		if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (dw->priv->file_list_store), &iter)) {
			GtkTreeSelection *selection;
			selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dw->priv->file_list));
			gtk_tree_selection_select_iter (selection, &iter);
    	}
		
		gchar *markup, *val1, *val2;
		glong size;
		
		g_object_get (entry, "name", &val1, NULL);
		markup = g_strdup_printf ("<b>%s</b>", val1);
		gtk_label_set_markup (GTK_LABEL (dw->priv->label_name), markup);
		g_free (val1);
		g_free (markup);
				
		g_object_get (entry, "version", &val1, NULL);
		g_object_get (entry, "release", &val2, NULL);
		markup = g_strdup_printf ("<b>%s</b>-%s", val1, val2);
		gtk_label_set_markup (GTK_LABEL (dw->priv->label_version), markup);
		g_free (val1);
		g_free (val2);
		g_free (markup);
		
		g_object_get (entry, "size", &size, NULL);
		markup = g_strdup_printf ("<b>%s</b>", parse_package_size (size));
		gtk_label_set_markup (GTK_LABEL (dw->priv->label_size), markup);
		g_free (markup);
		
		g_object_get (entry, "package-name", &val1, NULL);
		gtk_window_set_title (GTK_WINDOW (dw), val1);
		g_free (val1);
	}
	else {
		G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
	}
}

static void
on_btn_close_clicked_cb (GtkWidget *widget, GptDetailsWindow *dw)
{
	g_return_if_fail (GPT_IS_DETAILS_WINDOW (dw));
	
	gtk_list_store_clear (dw->priv->file_list_store);
	gtk_widget_destroy (GTK_WIDGET (dw));
}

GtkWidget *
gpt_details_window_new (PkgEntry *entry)
{
	GptDetailsWindow *w;
	
	g_return_val_if_fail (IS_PKG_ENTRY (entry), NULL);
	
	w = g_object_new (GPT_DETAILS_WINDOW_TYPE, "type", GTK_WINDOW_TOPLEVEL, "package", entry, NULL);
	
	return (GTK_WIDGET (w));
}

static gboolean
on_window_configure_event_cb (GtkWidget *widget, GdkEvent *event, gpointer data)
{
	g_return_val_if_fail (GPT_IS_DETAILS_WINDOW (widget), TRUE);
	
	gint w, h;	
	gtk_window_get_size (GTK_WINDOW (widget), &w, &h);
	g_object_set (the_app, "details-window-width", w, NULL);
	g_object_set (the_app, "details-window-height", h, NULL);
	
	return FALSE;
}

static gboolean
on_window_delete_event_cb (GtkWidget *widget, GdkEvent *event, gpointer data)
{
	g_return_val_if_fail (GPT_IS_DETAILS_WINDOW (widget), TRUE);
	
	gtk_list_store_clear (GPT_DETAILS_WINDOW (widget)->priv->file_list_store);
	gtk_widget_destroy (widget);
	
	return TRUE;
}
