/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <unistd.h>
#include <fcntl.h>
#include <gtk/gtk.h>

#include "progress_window.h"
#include "gpt.h"
#include "pkglist.h"
#include "misc.h"

#define TIMEOUT 100

static void gpt_progress_window_class_init (GptProgressWindowClass *klass);
static void gpt_progress_window_init (GptProgressWindow *pw);

static void gpt_progress_window_set_property (GObject           *obj,
                                              guint             prop_id,
                                              const GValue      *val,
                                              GParamSpec        *pspec);
static void gpt_progress_window_get_property (GObject           *obj,
                                              guint             prop_id,
                                              GValue            *val,
                                              GParamSpec        *pspec);

static gboolean on_window_delete_event_cb (GtkWidget    *widget,
                                           GdkEvent     *event,
                                           gpointer     data);
/* static gboolean on_window_configure_event_cb (GtkWidget *widget,
                                              GdkEvent  *event,
                                              gpointer  data); */
static void on_btn_close_clicked_cb (GtkWidget          *widget,
                                     GptProgressWindow  *pw);
static void on_btn_cancel_clicked_cb (GtkWidget         *widget,
                                      GptProgressWindow *pw);
static void on_expander_notify_cb (GObject              *object,
                                   GParamSpec           *param_spec,
                                   GptProgressWindow    *pw);
static gboolean on_timeout_cb (GptProgressWindow *pw);
static void on_child_exited_cb (VteReaper               *reaper,
                                gint                    child_pid,
                                gint                    retval,
                                GptProgressWindow       *pw);

static void on_adj_changed_cb (GtkWidget *adj, GptProgressWindow *pw);                                
static void on_adj_value_changed_cb (GtkWidget *adj, GptProgressWindow *pw);

extern GptApp *the_app;
static GtkWindowClass *parent_class = NULL;

static gboolean warn_on_failure = TRUE;
static gboolean scroll = TRUE;

enum
{
  PROP_MARKUP = 1,
  PROP_COMPLETE_MSG,
  PROP_FAILED_MSG,
  PROP_ABORTED_MSG,
};

GType
gpt_progress_window_get_type (void)
{
  static GType type = 0;

  if (type == 0)
    {
      static const GTypeInfo info =
        {
          sizeof (GptProgressWindowClass),
          NULL,   /* base_init */
          NULL,   /* base_finalize */
          (GClassInitFunc) gpt_progress_window_class_init,   /* class_init */
          NULL,   /* class_finalize */
          NULL,   /* class_data */
          sizeof (GptProgressWindow),
          0,      /* n_preallocs */
          (GInstanceInitFunc) gpt_progress_window_init    /* instance_init */
        };
		
      type = g_type_register_static (GTK_TYPE_WINDOW,
                                     "GptProgressWindow",
                                     &info, 0);
    }
  return type;
}

static void
gpt_progress_window_class_init (GptProgressWindowClass *klass)
{
  GObjectClass *obj_class;
  obj_class = G_OBJECT_CLASS (klass);
	
  parent_class = (GtkWindowClass *) g_type_class_peek_parent (klass);
  //obj_class->finalize = gpt_progress_window_finalize;
  obj_class->set_property = gpt_progress_window_set_property;
  obj_class->get_property = gpt_progress_window_get_property;

  g_object_class_install_property (obj_class, PROP_MARKUP,
                                   g_param_spec_string ("markup",
                                                         "Markup",
                                                         "Message markup",
                                                         "",
                                                         G_PARAM_READWRITE));	
  g_object_class_install_property (obj_class, PROP_COMPLETE_MSG,
                                   g_param_spec_string ("complete-message",
                                                         "Completed Message",
                                                         "Completed Message",
                                                         "",
                                                         G_PARAM_READWRITE));	
  g_object_class_install_property (obj_class, PROP_FAILED_MSG,
                                   g_param_spec_string ("failed-message",
                                                         "Failed Message",
                                                         "Failed Message",
                                                         "",
                                                         G_PARAM_READWRITE));	
  g_object_class_install_property (obj_class, PROP_ABORTED_MSG,
                                   g_param_spec_string ("aborted-message",
                                                         "Aborted Message",
                                                         "Aborted Message",
                                                         "",
                                                         G_PARAM_READWRITE));	
}

static void
gpt_progress_window_init (GptProgressWindow *pw)
{
  g_assert (GPT_IS_PROGRESS_WINDOW (pw));
	
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *icon;
  GtkWidget *vbox2;
  GtkWidget *label_exp;
  GtkWidget *hbox2;
  GtkWidget *hbox3;
  GtkWidget *hbox_btn;
  GtkWidget *btnbox;
	
  pw->priv = g_new0 (GptProgressWindowPriv, 1);
  g_return_if_fail (pw->priv != NULL);

  pw->priv->aborted_msg = g_strdup ("Operation aborted");
  pw->priv->complete_msg= g_strdup ("Operation completed succesfully");
  pw->priv->failed_msg = g_strdup ("Operation failed");
	
  gtk_container_set_border_width (GTK_CONTAINER (pw), 12);
  gtk_window_set_modal (GTK_WINDOW (pw), TRUE);
  gtk_window_set_skip_taskbar_hint (GTK_WINDOW (pw), TRUE);
  gtk_window_set_skip_pager_hint (GTK_WINDOW (pw), TRUE);
  gtk_window_set_type_hint (GTK_WINDOW (pw), GDK_WINDOW_TYPE_HINT_DIALOG);
	
  /* main vbox */
  vbox = gtk_vbox_new (FALSE, 6);
  gtk_widget_show (vbox);
  gtk_container_add (GTK_CONTAINER (pw), vbox);
	
  /* hbox for icon and message+expander */
  hbox = gtk_hbox_new (FALSE, 6);
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
	
  /* dialog icon */
  icon = gtk_image_new_from_stock ("gtk-dialog-info", GTK_ICON_SIZE_DIALOG);
  gtk_widget_show (icon);
  gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (icon), 0.5, 0);
  gtk_misc_set_padding (GTK_MISC (icon), 0, 6);
	
  /* vbox for message and expander */
  vbox2 = gtk_vbox_new (FALSE, 6);
  gtk_widget_show (vbox2);
  gtk_box_pack_start (GTK_BOX (hbox), vbox2, TRUE, TRUE, 0);
	
  /* dialog message label */
  pw->priv->label = gtk_label_new ("<big><b>Doing stuff</b></big>\n\nSomething is going on here right now. This may take quite some time.");
  gtk_widget_show (pw->priv->label);
  gtk_box_pack_start (GTK_BOX (vbox2), pw->priv->label, TRUE, TRUE, 0);
  gtk_label_set_use_markup (GTK_LABEL (pw->priv->label), TRUE);
  gtk_label_set_line_wrap (GTK_LABEL (pw->priv->label), TRUE);
  gtk_misc_set_alignment (GTK_MISC (pw->priv->label), 0, 0.5);
	
  /* "show details" expander */
  pw->priv->exp = gtk_expander_new (NULL);
  g_signal_connect (pw->priv->exp,
                    "notify::expanded",
                    G_CALLBACK (on_expander_notify_cb),
                    pw);
  gtk_widget_show (pw->priv->exp);
  gtk_box_pack_start (GTK_BOX (vbox2),
                      pw->priv->exp,
                      FALSE, TRUE, 0);
	
  /* expander label */
  label_exp = gtk_label_new ("Show details");
  gtk_widget_show (label_exp);
  gtk_expander_set_label_widget (GTK_EXPANDER (pw->priv->exp), label_exp);
	
  /* dummy expanding hbox */
  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (vbox),
                      hbox2,
                      TRUE, TRUE, 0);
	
  /* details textbox */
  pw->priv->details_frame = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_hide (pw->priv->details_frame);
  gtk_box_pack_start (GTK_BOX (hbox2),
                      pw->priv->details_frame,
                      TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (
                               GTK_SCROLLED_WINDOW (pw->priv->details_frame),
                               GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
  gtk_scrolled_window_set_shadow_type (
                               GTK_SCROLLED_WINDOW (pw->priv->details_frame),
                               GTK_SHADOW_IN);
  pw->priv->adj = GTK_WIDGET (gtk_scrolled_window_get_vadjustment (
                               GTK_SCROLLED_WINDOW (pw->priv->details_frame)));
  g_signal_connect (GTK_OBJECT (pw->priv->adj),
                    "changed",
                    G_CALLBACK (on_adj_changed_cb),
                    pw);
  g_signal_connect (GTK_OBJECT (pw->priv->adj),
                    "value-changed",
                    G_CALLBACK (on_adj_value_changed_cb),
                    pw);
                               
  pw->priv->details = gtk_text_view_new ();
  gtk_text_view_set_left_margin (GTK_TEXT_VIEW (pw->priv->details), 2);
  gtk_text_view_set_right_margin (GTK_TEXT_VIEW (pw->priv->details), 2);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (pw->priv->details),
                              FALSE);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (pw->priv->details),
                                    FALSE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (pw->priv->details),
                               GTK_WRAP_WORD_CHAR);
  gtk_widget_show (pw->priv->details);
  gtk_container_add (GTK_CONTAINER (pw->priv->details_frame),
                     pw->priv->details);
  pw->priv->details_buffer = gtk_text_buffer_new (NULL);
  gtk_text_view_set_buffer (GTK_TEXT_VIEW (pw->priv->details),
                            pw->priv->details_buffer);
                            
  /* hbox for buttons container */
  hbox_btn = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox_btn);
  gtk_box_pack_start (GTK_BOX (vbox),
                      hbox_btn,
                      FALSE, TRUE, 0);
	
  /* dummy expanding hbox */
  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (hbox_btn),
                      hbox3,
                      TRUE, TRUE, 0);
	
  /* invisible vte terminal */
  pw->priv->vte = VTE_TERMINAL (vte_terminal_new ());
  gtk_widget_hide (GTK_WIDGET (pw->priv->vte));
  gtk_box_pack_start (GTK_BOX (hbox3),
                      GTK_WIDGET (pw->priv->vte),
                      FALSE, FALSE, 0);
  pw->priv->reaper = vte_reaper_get ();
  g_signal_connect(G_OBJECT(pw->priv->reaper),
                   "child-exited",
                   G_CALLBACK(on_child_exited_cb),
                   pw);
	
  /* button box */
  btnbox = gtk_hbutton_box_new ();
  gtk_button_box_set_layout (GTK_BUTTON_BOX (btnbox),
                             GTK_BUTTONBOX_END);
  gtk_widget_show (btnbox);
  gtk_box_pack_start (GTK_BOX (hbox_btn),
                      btnbox,
                      FALSE, FALSE, 0);
	
  /* "close" button */
  pw->priv->btn_close = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
  gtk_widget_hide (pw->priv->btn_close);
  gtk_container_add (GTK_CONTAINER (btnbox),
                     pw->priv->btn_close);
  GTK_WIDGET_SET_FLAGS (pw->priv->btn_close,
                        GTK_CAN_DEFAULT);
  g_signal_connect (G_OBJECT (pw->priv->btn_close),
                    "clicked",
                    G_CALLBACK (on_btn_close_clicked_cb),
                    pw);
	
  pw->priv->btn_cancel = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
  gtk_widget_show (pw->priv->btn_cancel);
  gtk_container_add (GTK_CONTAINER (btnbox),
                     pw->priv->btn_cancel);
  GTK_WIDGET_SET_FLAGS (pw->priv->btn_cancel,
                        GTK_CAN_DEFAULT);
  g_signal_connect (G_OBJECT (pw->priv->btn_cancel),
                    "clicked",
                    G_CALLBACK (on_btn_cancel_clicked_cb),
                    pw);

  pw->priv->pid = 0;
  pw->priv->log_fd = 0;
	
  pw->priv->temp_dir = g_strdup (GPT_BUILD_DIR);
  if (mkdtemp (pw->priv->temp_dir) == NULL)
    {
      gpt_error (GTK_WINDOW (pw), "Could not create a temporary directory");
      exit (1);	/* FIXME: this is really rude... */
    }
	
  g_signal_connect (G_OBJECT (pw),
                    "delete-event",
                    G_CALLBACK (on_window_delete_event_cb),
                    NULL);
  gtk_window_set_resizable (GTK_WINDOW (pw), FALSE);
}

GtkWidget *
gpt_progress_window_new (void)
{
  return GTK_WIDGET (g_object_new (GPT_PROGRESS_WINDOW_TYPE,
                                   "type", GTK_WINDOW_TOPLEVEL,
                                   NULL));
}

static void
gpt_progress_window_set_property (GObject       *obj,
                                  guint         prop_id,
                                  const         GValue *val,
                                  GParamSpec    *pspec)
{
  GptProgressWindow *pw;
  pw = GPT_PROGRESS_WINDOW (obj);
	
  gchar *str;
	
  switch (prop_id)
    {
      case PROP_MARKUP:
        str = g_strdup (g_value_get_string (val));
        gtk_label_set_markup (GTK_LABEL (pw->priv->label), str);
        g_free (str);
        break;
      case PROP_COMPLETE_MSG:
        g_free (pw->priv->complete_msg);
        pw->priv->complete_msg = g_strdup (g_value_get_string (val));
        break;
      case PROP_ABORTED_MSG:
        g_free (pw->priv->aborted_msg);
        pw->priv->aborted_msg = g_strdup (g_value_get_string (val));
        break;
      case PROP_FAILED_MSG:
        g_free (pw->priv->failed_msg);
        pw->priv->failed_msg = g_strdup (g_value_get_string (val));
        break;
      default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
        break;
    }
}

static void
gpt_progress_window_get_property (GObject     *obj,
                                  guint       prop_id,
                                  GValue      *val,
                                  GParamSpec  *pspec)
{
  GptProgressWindow *pw;
  pw = GPT_PROGRESS_WINDOW (obj);
	
  gchar *str;
	
  switch (prop_id)
    {
      case PROP_MARKUP:
        str = gtk_label_get_label (GTK_LABEL (pw->priv->label));
        g_value_set_string (val, str);
        break;
      case PROP_COMPLETE_MSG:
        g_value_set_string (val, pw->priv->complete_msg);
        break;
      case PROP_ABORTED_MSG:
        g_value_set_string (val, pw->priv->aborted_msg);
        break;
      case PROP_FAILED_MSG:
        g_value_set_string (val, pw->priv->failed_msg);
        break;
      default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
        break;
    }
}

void
gpt_progress_window_run (GptProgressWindow  *pw,
                         const gchar        *cmd)
{
  g_return_if_fail (GPT_IS_PROGRESS_WINDOW (pw));
  g_return_if_fail (cmd != NULL);

  g_message ("gpt_progress_window_run(): %s\n", cmd);
	
  gchar *args[6];

  if (the_app->need_auth)
    {
      pw->priv->auth_done = FALSE;
      args [0] = g_strdup ("su");
    }
  else
    {
      pw->priv->auth_done = TRUE;
      args [0] = g_strdup ("bash");
    }
	
  args [1] = "-c";
  args [2] = g_strdup_printf ("%s > progress.log 2>&1", cmd);
  args [3] = NULL;
	
  pw->priv->log_fd = 0;

  warn_on_failure = TRUE;
  pw->priv->pid = vte_terminal_fork_command (VTE_TERMINAL(pw->priv->vte),
                                             args[0], args,
                                             NULL, pw->priv->temp_dir,
                                             TRUE, TRUE, TRUE);

  g_timeout_add(TIMEOUT, (gpointer) on_timeout_cb, pw);
	
  gtk_widget_show (GTK_WIDGET (pw));
}

static void
on_btn_close_clicked_cb (GtkWidget *widget, GptProgressWindow *pw)
{
  g_return_if_fail (GPT_IS_PROGRESS_WINDOW (pw));
//  g_return_if_fail (pw->priv->pid > 0);	/* FIXME */

  if (gtk_expander_get_expanded (GTK_EXPANDER (pw->priv->exp)))
    {
      gint w, h;
      gtk_window_get_size (GTK_WINDOW (pw), &w, &h);
      g_object_set (the_app, "progress-window-width", w, NULL);
      g_object_set (the_app, "progress-window-height", h, NULL);
    }
	
  gtk_widget_destroy (GTK_WIDGET (pw));
}

static void
on_btn_cancel_clicked_cb (GtkWidget *widget, GptProgressWindow *pw)
{
  g_return_if_fail (GPT_IS_PROGRESS_WINDOW (pw));
	
  GtkWidget *dlg = gtk_message_dialog_new (GTK_WINDOW(pw),
                                           GTK_DIALOG_DESTROY_WITH_PARENT,
		                           GTK_MESSAGE_QUESTION,
		                           GTK_BUTTONS_OK_CANCEL,
		                           "Abort operation?");
	
  gint response = gtk_dialog_run (GTK_DIALOG (dlg));
  gtk_widget_destroy (dlg);
	
  if (response == GTK_RESPONSE_OK)
    {
      if (the_app->need_auth)
        {
          /* we need to SIGTERM the child as root */
          gchar *args[4];
          args[0] = g_strdup ("su");
          args[1] = g_strdup ("-c");
          args[2] = g_strdup_printf ("kill -TERM %d", pw->priv->pid);
          args[3] = NULL;
                  
          GtkWidget *t2;
          t2 = vte_terminal_new ();
          vte_terminal_fork_command (VTE_TERMINAL (t2),
                                     args[0], args,
                                     NULL, NULL,
                                     TRUE, TRUE, TRUE);
          usleep(500000);
          vte_terminal_feed_child (VTE_TERMINAL (t2),
                                   the_app->root_password,
                                   strlen (the_app->root_password));
          vte_terminal_feed_child (VTE_TERMINAL (t2),
                                   "\n", 1);
        }
      else
          kill (pw->priv->pid, SIGKILL);
		
      warn_on_failure = FALSE;
    }
}

static gboolean
on_window_delete_event_cb (GtkWidget *widget, GdkEvent *event, gpointer data)
{
  g_return_val_if_fail (GPT_IS_PROGRESS_WINDOW (widget), TRUE);
//  g_return_val_if_fail (GPT_PROGRESS_WINDOW (widget)->priv->pid > 0, FALSE);	/* FIXME */
	
  if (gtk_expander_get_expanded (GTK_EXPANDER (
                                    GPT_PROGRESS_WINDOW (widget)->priv->exp)))
    {
      gint w, h;
      gtk_window_get_size (GTK_WINDOW (widget), &w, &h);
      g_object_set (the_app, "progress-window-width", w, NULL);
      g_object_set (the_app, "progress-window-height", h, NULL);
    }

  gtk_widget_destroy (widget);

  return TRUE;
}

static void
on_expander_notify_cb (GObject            *object,
                       GParamSpec         *param_spec,
                       GptProgressWindow  *pw)
{
  g_return_if_fail (GPT_IS_PROGRESS_WINDOW (pw));
	
  gint w, h;
	
  GtkExpander *expander;
  expander = GTK_EXPANDER (object);
	
  if (gtk_expander_get_expanded (expander))
    {
      gtk_widget_show_all (pw->priv->details_frame);
		
      g_object_get (the_app, "progress-window-width", &w, NULL);
      g_object_get (the_app, "progress-window-height", &h, NULL);
      gtk_window_set_resizable (GTK_WINDOW (pw), TRUE);		
      gtk_window_resize (GTK_WINDOW (pw), w, h);
    }
  else
    {
      /* save expanded window size */
      gtk_window_get_size (GTK_WINDOW (pw), &w, &h);
      g_object_set (the_app, "progress-window-width", w, NULL);
      g_object_set (the_app, "progress-window-height", h, NULL);
		
      gtk_widget_hide_all (pw->priv->details_frame);
      gtk_window_resize (GTK_WINDOW (pw), 1, 1);
      gtk_window_set_resizable (GTK_WINDOW (pw), FALSE);
    }
}

static void
on_child_exited_cb (VteReaper           *reaper,
                    gint                child_pid,
                    gint                retval,
                    GptProgressWindow   *pw)
{
  g_return_if_fail (GPT_IS_PROGRESS_WINDOW (pw));
	
  if(pw->priv->pid == 0)
          g_critical("Received SIGCHLD from process %d, but no child process is running",
                     child_pid);
	
  else if(child_pid == pw->priv->pid)
    {
      g_message("Child process %d returned %d", child_pid, retval);
      pw->priv->pid = 0;
		
      gtk_widget_hide (pw->priv->btn_cancel);
      gtk_widget_show (pw->priv->btn_close);

      if (retval && warn_on_failure)
          gpt_error (GTK_WINDOW (pw), pw->priv->failed_msg);
      
      else if (retval == 0)
          gpt_info (GTK_WINDOW (pw), pw->priv->complete_msg);
      
      g_signal_handlers_disconnect_by_func (G_OBJECT (reaper),
                                            G_CALLBACK (on_child_exited_cb),
                                            pw);
    }

  else
    {
      g_warning("Ignoring SIGCHLD from process %d", child_pid);
    }

  packages_list_refresh (PKG_LIST (the_app->main_window->priv->packages_list),
                         NULL);  
}

static gboolean
on_timeout_cb (GptProgressWindow *pw)
{
  gboolean retval;
	
  if(pw->priv->pid) 
      retval = TRUE;
  else
      retval = FALSE;
	
  /* we may need to give root password here */
  if (! pw->priv->auth_done)
    {
      gchar *str;
      str = vte_terminal_get_text_range (VTE_TERMINAL (pw->priv->vte),
                                         0, 0, 0, 16, vte_is_selected,
                                         NULL, NULL);
      if(strncmp(&str[1], "assword:", strlen("assword:")) == 0)
        {
          vte_terminal_feed_child (VTE_TERMINAL (pw->priv->vte),
                                   the_app->root_password,
                                   strlen(the_app->root_password));
          vte_terminal_feed_child (VTE_TERMINAL (pw->priv->vte),
                                   "\n", 1);
          pw->priv->auth_done = TRUE;
        }
      g_free(str);
    }
	
  /* (re-)try to open the logfile if needed*/
  if (pw->priv->log_fd < 1)
    { 
      gchar *fname = g_strdup_printf ("%s/progress.log", pw->priv->temp_dir);
      pw->priv->log_fd = open (fname, O_RDONLY|O_NONBLOCK);
      if (pw->priv->log_fd == -1)
          perror ("open");
      g_free (fname);
    }

  if (pw->priv->log_fd > 0)
    {
      ssize_t n;
      gchar *str = g_new0 (gchar, 65);
      GtkTextIter iter;
      do {
          n = read (pw->priv->log_fd, str, 64);
          if (n > 0)
            {
              gtk_text_buffer_get_end_iter (pw->priv->details_buffer,
                                            &iter);
              gtk_text_buffer_insert (pw->priv->details_buffer,
                                      &iter, str, n);
            }
          else if (n == -1)
              perror ("read");
      } while (n > 0);
		
      /* close the logfile if the child process has already finished */
      if (!retval)
        {
          close (pw->priv->log_fd);
          pw->priv->log_fd = 0;
			
          if (!warn_on_failure)
            {
              /* the operation has been aborted by user */
              gtk_text_buffer_get_end_iter (pw->priv->details_buffer,
                                            &iter);
              gtk_text_buffer_insert (pw->priv->details_buffer,
                                      &iter,
                                      "\nAborted by user\n",
                                      -1);
            }
        }
    }

  return retval;
}

static void
on_adj_changed_cb (GtkWidget *adj, GptProgressWindow *pw)
{
  g_return_if_fail (GTK_IS_ADJUSTMENT (adj));
  g_return_if_fail (GPT_IS_PROGRESS_WINDOW (pw));

  gdouble value, upper, page;
  g_object_get (adj, "page-size", &page, "upper", &upper, "value", &value, NULL);
  
  if (scroll)
      g_object_set (adj, "value", upper - page, NULL);
}

static void
on_adj_value_changed_cb (GtkWidget *adj, GptProgressWindow *pw)
{
  g_return_if_fail (GTK_IS_ADJUSTMENT (adj));
  g_return_if_fail (GPT_IS_PROGRESS_WINDOW (pw));

  gdouble value, upper, page;
  g_object_get (adj, "page-size", &page, "upper", &upper, "value", &value, NULL);
  
  if (value >= upper - page)
      scroll = TRUE;
  else
      scroll = FALSE;
}
